SystemOrganization addCategory: #'HTTPClient-Methods'!
SystemOrganization addCategory: #'HTTPClient-Network'!
SystemOrganization addCategory: #'HTTPClient-Tests'!
SystemOrganization addCategory: #'HTTPClient-Session'!
SystemOrganization addCategory: #'HTTPClient-Core'!

Stream subclass: #HCBufferedStream
	instanceVariableNames: 'buffer startPos endPos'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Network'!

!HCBufferedStream class methodsFor: 'instance creation' stamp: 'cc 5/4/2004 23:27'!
on: aString
	^self basicNew on: aString! !

!HCBufferedStream methodsFor: 'accessing' stamp: 'cc 5/4/2004 23:27'!
contents
	"return contents, non-destructive"
	^buffer copyFrom: startPos to: endPos! !

!HCBufferedStream methodsFor: 'testing' stamp: 'cc 5/4/2004 23:27'!
includesAll: aString
	"answer whether the buffer contains aString"
	| tmp |
	tmp _ buffer findString: aString startingAt: startPos.
	^(tmp >= startPos) and: [tmp <= endPos]! !

!HCBufferedStream methodsFor: 'testing' stamp: 'cc 5/4/2004 23:27'!
includes: aChar
	"answer whether buffer includes aChar within bounds"
	| tmp |
	tmp _ buffer indexOf: aChar startingAt: startPos.
	^(tmp >= startPos) and: [tmp <= endPos]! !

!HCBufferedStream methodsFor: 'testing' stamp: 'cc 5/4/2004 23:27'!
isEmpty
	^self size <= 0! !

!HCBufferedStream methodsFor: 'accessing' stamp: 'cc 5/4/2004 23:27'!
match: aString
	self size < aString size
		ifTrue: [^false].
	(self next: aString size) = aString
		ifTrue: [^true]
		ifFalse: [self on: aString, self upToEnd].
	^false! !

!HCBufferedStream methodsFor: 'accessing' stamp: 'cc 5/4/2004 23:27'!
next
	"return the next character"
	"consume it"
	^buffer at: (startPos _ startPos + 1) - 1! !

!HCBufferedStream methodsFor: 'accessing' stamp: 'cc 5/4/2004 23:27'!
nextPutAll: aString
	buffer _ (buffer copyFrom: startPos to: endPos), aString.
	startPos _ 1.
	endPos _ buffer size! !

!HCBufferedStream methodsFor: 'accessing' stamp: 'cc 5/4/2004 23:27'!
next: anInteger
	"return next anInteger characters"
	"consumes them"
	^buffer copyFrom: startPos to: (startPos _ startPos + anInteger) - 1! !

!HCBufferedStream methodsFor: 'initialize-release' stamp: 'cc 5/4/2004 23:27'!
on: aString
	buffer _ aString.
	startPos _ 1.
	endPos _ buffer size! !

!HCBufferedStream methodsFor: 'accessing' stamp: 'cc 5/4/2004 23:27'!
peek
	"return the next character without consuming it"
	"consume it"
	^buffer at: startPos! !

!HCBufferedStream methodsFor: 'printing' stamp: 'cc 5/4/2004 23:27'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: '(', self size asString, ')'! !

!HCBufferedStream methodsFor: 'initialize-release' stamp: 'cc 5/4/2004 23:27'!
reset
	"hack"
	self on: ''! !

!HCBufferedStream methodsFor: 'accessing' stamp: 'cc 5/4/2004 23:27'!
size
	^endPos - startPos + 1! !

!HCBufferedStream methodsFor: 'accessing' stamp: 'cc 5/4/2004 23:27'!
upToAll: aString
	"return a string before the occurence of aString, if any"
	"consume it"
	| i |
	i _ buffer findString: aString startingAt: startPos.
	(i <= 0 or: [i > endPos])
		ifTrue: [^self upToEnd].
	^self next: (i-startPos)! !

!HCBufferedStream methodsFor: 'accessing' stamp: 'cc 5/4/2004 23:27'!
upToEnd
	"return buffer contents,
	consume all my buffer"
	| string |
	string _ self contents.
	self reset.
	^string! !

!HCBufferedStream methodsFor: 'accessing' stamp: 'cc 5/4/2004 23:27'!
upTo: aChar
	"return String up to aChar, but not including"
	"consumes the returned string"
	| i |
	i _ buffer indexOf: aChar startingAt: startPos.
	(i <= 0 or: [i > endPos])
		ifTrue: [^self upToEnd].
	^self next: (i-startPos)! !

Stream subclass: #HCSocketStream
	instanceVariableNames: 'socket inStream outStream inDataCount outDataCount autoFlush binary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Network'!

!HCSocketStream commentStamp: '<historical>' prior: 0!
Kom48 (Comanche beta 4.8)
bolot 4/2/2001 18:47

HTTPSocketStream
- some changes to gradually enable client-side work

Comment from kom46:
HTTPSocketStream
- stream interface to Socket
- HttpRequest (and other users) don't know anything about sockets

bolot 12/16/1999 15:01
- re-doing the interface
  - inStream will replace buffer
  - outStream will be the buffer for sending data
  - nextPut:/nextPutAll: will write to outStream
  - flush will be required to send data
- TODO
  - stop using buffer, eventually remove it
!

HCSocketStream subclass: #HCConnection
	instanceVariableNames: 'keepalive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Network'!

!HCConnection commentStamp: 'cc 5/5/2004 00:31' prior: 0!
I contain the HTTP Connection Stream, and also fix some things in SocketStream (rather than changing the class itself).

I do nifty things like KeepAlive.!

!HCConnection class methodsFor: 'as yet unclassified' stamp: 'cc 5/4/2004 05:31'!
connectToHost: host port: portNumber
	Socket initializeNetwork.
	^ self on: (Socket new connectToHostNamed: host port: portNumber)! !

!HCConnection methodsFor: 'as yet unclassified' stamp: 'cc 1/19/2005 21:00'!
defaultTimeout
	"300 seconds for keepalive timeout"
	^ 300! !

!HCConnection methodsFor: 'as yet unclassified' stamp: 'cc 1/16/2005 19:49'!
ifStale: aBlock
	self isStale ifTrue: aBlock! !

!HCConnection methodsFor: 'as yet unclassified' stamp: 'cc 5/4/2004 05:15'!
isFresh
	^ self socket isValid and: [self keepalive < TimeStamp current]! !

!HCConnection methodsFor: 'as yet unclassified' stamp: 'cc 5/4/2004 05:15'!
isStale
	^ self isFresh not! !

!HCConnection methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 19:59'!
keepalive
	^ keepalive ifNil: [ keepalive _ TimeStamp current plusSeconds: self defaultTimeout ]! !

!HCConnection methodsFor: 'as yet unclassified' stamp: 'cc 1/16/2005 20:10'!
sendCommand: aCommand
	"Send the command and flush it".
	self nextPutAll: aCommand; flush.! !

!HCSocketStream class methodsFor: 'instance creation' stamp: 'bolot 12/16/1999 18:06'!
on: socket
	"create a socket stream on a server socket"
	"for HTTP style header must call readHeader explicitly"
	^self basicNew initialize: socket! !

!HCSocketStream methodsFor: 'testing' stamp: 'cc 1/19/2005 21:23'!
atEnd
	^self isConnected not & inStream isEmpty & self isDataAvailable not! !

!HCSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 17:02'!
autoFlushOff
	"Output is sent only when flush method is called"
	autoFlush _ false! !

!HCSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 17:06'!
autoFlushOn
	"Have nextPutAll:, cr, crlf flush output buffer"
	autoFlush _ true! !

!HCSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 13:35'!
binary
	"Set the stream to read/write binary data"
	self setBinary: true! !

!HCSocketStream methodsFor: 'private' stamp: 'rew 5/18/2001 16:40'!
bufferClass
	^self isBinary
		ifTrue:[ByteArray]
		ifFalse:[String]! !

!HCSocketStream methodsFor: 'private' stamp: 'rew 5/18/2001 16:40'!
bufferSized: anInteger
	^self bufferClass new: anInteger! !

!HCSocketStream methodsFor: 'initialize-release' stamp: 'bolot 2/5/2001 15:46'!
close
	self flush.
	self socket closeAndDestroy: 30! !

!HCSocketStream methodsFor: 'accessing' stamp: 'cc 1/19/2005 21:20'!
closeAfter: aBlock
	aBlock ensure: [self close]! !

!HCSocketStream methodsFor: 'accessing' stamp: 'cc 1/20/2005 00:01'!
contents: aLength
	^ self next: aLength! !

!HCSocketStream methodsFor: 'stream out' stamp: 'bolot 12/27/1999 12:17'!
cr
	self nextPutAll: String cr! !

!HCSocketStream methodsFor: 'stream out' stamp: 'bolot 8/11/1999 22:13'!
crlf
	self nextPutAll: String crlf! !

!HCSocketStream methodsFor: 'stream out' stamp: 'rew 5/21/2001 12:22'!
flush
	socket isOtherEndClosed
		ifFalse: [socket sendData: (outStream contents)].
	"isOtherEndConnected does not exist in 2.8 image"
	"socket isOtherEndConnected
		ifTrue: [socket sendData: outStream contents]."
	self incOutDataCountBy: outStream contents size.
	self resetOutStream! !

!HCSocketStream methodsFor: 'private' stamp: 'cc 1/19/2005 22:11'!
for: timeLimit do: aBlock every: interval while: booleanBlock
	| end |
	end _ TimeStamp current plusSeconds: timeLimit.
	
	[booleanBlock value & (TimeStamp current < end)]
		whileTrue: [ 
			aBlock value.
			(Delay forMilliseconds: interval) wait]! !

!HCSocketStream methodsFor: 'statistics' stamp: 'bolot 12/16/2000 03:03'!
inBytes "actual number of bytes read by the app"
	^self inDataCount - inStream size! !

!HCSocketStream methodsFor: 'statistics' stamp: 'rew 5/18/2001 17:09'!
incInDataCountBy: increment
	"increment inDataCount by"
	inDataCount _ inDataCount + increment! !

!HCSocketStream methodsFor: 'statistics' stamp: 'rew 5/18/2001 17:09'!
incOutDataCountBy: increment
	"increment outDataCount by"
	outDataCount _ outDataCount + increment! !

!HCSocketStream methodsFor: 'statistics' stamp: 'rew 5/18/2001 17:08'!
inDataCount
	^inDataCount! !

!HCSocketStream methodsFor: 'initialize-release' stamp: 'rew 5/18/2001 17:03'!
initialize
	self resetStatus.
	self autoFlushOn.
	self text.
	self resetInStream.
	self resetOutStream! !

!HCSocketStream methodsFor: 'initialize-release' stamp: 'bolot 12/16/2000 01:46'!
initialize: aSocket
	self initialize.
	self socket: aSocket! !

!HCSocketStream methodsFor: 'testing' stamp: 'rew 5/18/2001 17:05'!
isAutoFlushOn
	^autoFlush! !

!HCSocketStream methodsFor: 'testing' stamp: 'rew 5/18/2001 17:11'!
isBinary
	^binary! !

!HCSocketStream methodsFor: 'testing' stamp: 'bolot 2/5/2001 15:47'!
isConnected
	^self socket isConnected! !

!HCSocketStream methodsFor: 'testing' stamp: 'bolot 2/5/2001 15:47'!
isDataAvailable
	^self socket dataAvailable! !

!HCSocketStream methodsFor: 'testing' stamp: 'bolot 2/5/2001 15:48'!
isOtherEndConnected
	^self socket isOtherEndConnected! !

!HCSocketStream methodsFor: 'testing' stamp: 'rew 5/18/2001 13:23'!
isText
	^self isBinary not! !

!HCSocketStream methodsFor: 'stream out' stamp: 'rew 5/18/2001 18:09'!
lf
	self nextPutAll: (String with: Character lf)! !

!HCSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 16:45'!
localAddress
	^socket localAddress! !

!HCSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 16:45'!
localPort
	^socket localPort! !

!HCSocketStream methodsFor: 'stream in' stamp: 'bolot 12/16/2000 03:04'!
match: aString
	"position past the next occurrence of aString"
	"bolot 12/13/2000 00:49 - must begin with it"
	"answer if aString is found"
	inStream size < aString size
		ifTrue: [self flag: #todo. "should try to read in more, unless #atEnd"
			"but since atEnd is somewhat broken, don't use it yet"
			^false].

	(inStream contents beginsWith: aString)
		ifTrue: [inStream next: aString size.
			^true].

	^false! !

!HCSocketStream methodsFor: 'stream in' stamp: 'bolot 1/4/2000 20:31'!
next
	self atEnd ifTrue: [^nil].
	inStream isEmpty ifTrue: [self pvtGetData].
	^inStream next! !

!HCSocketStream methodsFor: 'stream in' stamp: 'bolot 1/5/2000 15:59'!
nextChunk
	"getData front - retrieve all data available now"
	inStream isEmpty
		ifTrue: [self pvtGetData].
	^inStream upToEnd! !

!HCSocketStream methodsFor: 'stream out' stamp: 'rew 5/18/2001 17:07'!
nextPutAll: string
	outStream nextPutAll: string.
	self isAutoFlushOn ifTrue: [self flush]! !

!HCSocketStream methodsFor: 'stream out' stamp: 'rew 5/19/2001 11:24'!
nextPut: char
	self nextPutAll: (self bufferClass with: char)! !

!HCSocketStream methodsFor: 'stream in' stamp: 'cc 1/20/2005 00:06'!
next: aLength
	"Wait for data up to aLength, disconnection, or timeout (30 sec)"
	self
		for: 30
		do: [self isDataAvailable ifTrue: [self pvtGetData]]
		every: 200
		while: [self isConnected & inStream size < aLength].

	^inStream next: aLength
! !

!HCSocketStream methodsFor: 'statistics' stamp: 'rew 5/18/2001 17:10'!
outDataCount
	^outDataCount! !

!HCSocketStream methodsFor: 'stream in' stamp: 'bolot 12/22/1999 17:06'!
putDataBack: string
	| tempString |
	tempString _ inStream contents.
	self resetInStream.
	inStream nextPutAll: string; nextPutAll: tempString! !

!HCSocketStream methodsFor: 'private' stamp: 'cc 1/19/2005 23:01'!
pvtGetData
	| data buffer bytesRead|
	buffer _ self bufferSized: 4000.
	(socket isConnected and: [socket dataAvailable not])
		ifTrue:[socket waitForDataFor: 200].
	bytesRead _ socket receiveDataInto: buffer.
	data _ buffer
			copyFrom: 1
			to: bytesRead.
	inStream nextPutAll: data.
	self incInDataCountBy: bytesRead.
	^data! !

!HCSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 16:45'!
remoteAddress
	^socket remoteAddress! !

!HCSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 16:46'!
remotePort
	^socket remotePort! !

!HCSocketStream methodsFor: 'initialize-release' stamp: 'cc 5/5/2004 00:21'!
resetInStream
	inStream _ HCBufferedStream on: (self bufferSized: 0)! !

!HCSocketStream methodsFor: 'initialize-release' stamp: 'cc 1/19/2005 21:19'!
resetInStreamAfter: aBlock
	aBlock ensure: [self resetInStream]! !

!HCSocketStream methodsFor: 'initialize-release' stamp: 'cc 5/5/2004 00:21'!
resetOutStream
	outStream _ HCBufferedStream on: (self bufferSized: 0)! !

!HCSocketStream methodsFor: 'initialize-release' stamp: 'cc 1/19/2005 21:20'!
resetOutStreamAfter: aBlock
	aBlock ensure: [self resetOutStream]! !

!HCSocketStream methodsFor: 'initialize-release' stamp: 'rew 5/18/2001 16:43'!
resetStatus
	inDataCount _ 0.
	outDataCount _ 0! !

!HCSocketStream methodsFor: 'private' stamp: 'rew 5/18/2001 17:11'!
setBinary: aBoolean
	"Set the stream to handle binary or text"
	binary _ aBoolean.
	self resetInStream.
	self resetOutStream! !

!HCSocketStream methodsFor: 'accessing' stamp: 'bolot 2/5/2001 15:44'!
socket
	^socket! !

!HCSocketStream methodsFor: 'accessing' stamp: 'cc 1/10/2005 15:32'!
socketstream
	^self! !

!HCSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 16:47'!
socket: aSocket
	socket _ aSocket.
! !

!HCSocketStream methodsFor: 'accessing' stamp: 'rew 5/18/2001 13:36'!
text
	"Set the stream to read/write text data"
	self setBinary: false! !

!HCSocketStream methodsFor: 'stream in' stamp: 'cc 5/5/2004 15:57'!
upToAll: delims
	| string |
	(inStream includesAll: delims)
		ifTrue: ["Found the delimiter part way into buffer"
			string _ inStream upToAll: delims.
			inStream next: delims size.
			^string].
	self isConnected not & self isDataAvailable not
		ifTrue: ["Never found it, and hit end of file"
			^inStream upToEnd].
	"Never found it, but there's more..."
	self pvtGetData.
	^self upToAll: delims! !

!HCSocketStream methodsFor: 'stream in' stamp: 'cc 1/19/2005 22:05'!
upToEnd
	[self isConnected or: [self isDataAvailable]]
		whileTrue:[self pvtGetData].
	self resetInStreamAfter: [^inStream contents]
! !

!HCSocketStream methodsFor: 'stream in' stamp: 'rew 5/20/2001 13:29'!
upTo: delim 
	| string |
	(inStream includes: delim)
		ifTrue: ["Found the delimiter part way into buffer"
			string _ inStream upTo: delim.
			inStream next.
			^string].
	self isConnected not & self isDataAvailable not
		ifTrue: ["Never found it, and hit end of file"
			string _ inStream contents.
			self resetInStream.
			^string].
	"Never found it, but there's more..."
	self pvtGetData.
	^self upTo: delim! !

!HierarchicalUrl methodsFor: '*HTTPClient' stamp: 'cc 5/15/2004 07:13'!
page
	| last page |
	last _ self path last.
	page _ self pathstring.

	(last = '') ifFalse: [page _ page, '/', self path last].

	^ page! !

!HierarchicalUrl methodsFor: '*HTTPClient' stamp: 'cc 5/7/2004 21:43'!
pageAndQuery
	^ self page, (self query ifNotNil: ['?', self query] ifNil: ['']).! !

!HierarchicalUrl methodsFor: '*HTTPClient' stamp: 'cc 5/8/2004 00:45'!
pathstring
	| ws dirs |
	ws _ WriteStream on: String new.
	ws nextPut: $/.
	dirs _ self path select: [:e | (e = self path last) not].
	dirs
		do: [:pathelem | ws nextPutAll: pathelem] 
		separatedBy: [ws nextPut: $/].
	(self path last = '' and: [(self path first = '') not]) 
		ifTrue: [ws nextPut: $/].
	^ ws contents! !

Object subclass: #HCAuth
	instanceVariableNames: 'realm url user password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Session'!

!HCAuth class methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 23:39'!
fromType: aType
	self allSubclassesDo: [:sub | 
		sub type = aType ifTrue: [ ^sub new ]
	].
	^ nil! !

!HCAuth class methodsFor: 'as yet unclassified' stamp: 'cc 5/10/2004 22:16'!
isValidType: aType
	^ self allSubclasses anySatisfy: [:sub | aType = sub type].! !

!HCAuth class methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 23:36'!
type
	self subclassResponsibility! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:54'!
acceptUrl: aUrl
	self subclassResponsibility! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 22:08'!
fromPending: pendingAuth info: infoDict
	"Sets the info of this authentication token to that of the pending token
	and the information dictionary"
	self subclassResponsibility! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 06:51'!
max: aToken
	"Implemented for selecting the most specific token"
	^ self < aToken 
		ifTrue: [aToken]
		ifFalse: [self]! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:45'!
password
	^ password! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:45'!
password: aPassword
	password _ aPassword! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:31'!
realm
	^ realm! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:31'!
realm: aRealm
	realm _ aRealm.! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/9/2004 20:14'!
renderHeaderOn: aWriteStream forMethod: aMethod
	self subclassResponsibility! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 03:35'!
updateAuthInfo: infoDict
	"Update yourself from the authorization-info header that was sent."
	self subclassResponsibility! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/9/2004 18:45'!
updateFromInfo: infoDict
	"Update yourself from the information dictionary that was just sent"
	self subclassResponsibility! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:48'!
url
	^ url! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:49'!
url: aUrl
	url _ aUrl! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:44'!
user
	^ user! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:44'!
user: aUser
	user _ aUser! !

!HCAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 06:51'!
< aToken
	"Implemented for selecting the most specific token"
	^ self url path size < aToken url path size! !

HCAuth subclass: #HCBasicAuth
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Session'!

!HCBasicAuth class methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 23:36'!
type
	^ 'Basic'! !

!HCBasicAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 20:03'!
acceptUrl: aUrl
	^ aUrl pathstring includesSubString: self url pathstring! !

!HCBasicAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 19:45'!
encodedUserPass
	^ (Base64MimeConverter mimeEncode: (self user, ':', self password) readStream)! !

!HCBasicAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/8/2004 00:12'!
fromPending: authToken info: infoDict
	self
		url: authToken url;
		user: authToken user;
		password: authToken password;
		realm: (infoDict at: 'realm' ifAbsent: [nil]).! !

!HCBasicAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/9/2004 20:14'!
renderHeaderOn: aWriteStream forMethod: aMethod
	aWriteStream 
		nextPutAll: self class type;
		nextPutAll: ' ';
		nextPutAll: self encodedUserPass.
	! !

!HCBasicAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 03:35'!
updateAuthInfo: infoDict
	self shouldNotImplement! !

!HCBasicAuth methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 04:42'!
updateFromInfo: infoDict
	self realm: (infoDict at: 'realm' ifAbsent: [^nil]).! !

HCAuth subclass: #HCDigestAuth
	instanceVariableNames: 'method uri domain cnonce nonce nonceCount opaque algorithm qop'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Session'!

!HCDigestAuth class methodsFor: 'as yet unclassified' stamp: 'cc 5/10/2004 00:00'!
type
	^ 'Digest'! !

!HCDigestAuth methodsFor: 'auth accessing' stamp: 'cc 5/9/2004 19:22'!
acceptUrl: aUrl
	^ self domain anySatisfy: [:dom | 
		aUrl pathstring includesSubString: dom pathstring]! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:03'!
algorithm
	^ algorithm ifNil: [ algorithm _ 'MD5' ]! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:05'!
algorithm: anAlgorithm
	| supported |
	supported _ #('MD5' 'MD5-sess').

	(supported includes: anAlgorithm) 
		ifTrue: [algorithm _ anAlgorithm].! !

!HCDigestAuth methodsFor: 'authentication info' stamp: 'cc 5/11/2004 04:27'!
authorizeRspauth: rspauth qop: aQop
	| a2 |
	a2 _ ':', self uri.
	aQop = 'auth-int' ifTrue: [a2 _ a2, (self hash: self method entityBody)].

	^ (self requestDigestA1: self getA1 A2: a2) = rspauth
	! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/10/2004 17:40'!
cnonce
	| from to |
	^ cnonce ifNil: [
		from _ self hash: (self nonce, ':', TimeStamp current asSeconds asString).
		to _ WriteStream on: String new.
		8 timesRepeat: [to nextPut: from atRandom].
		cnonce _ to contents asLowercase.
	]! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/10/2004 00:20'!
cnonceHeaderOn: header
	self qop ifNil: [^nil].
	self tabsOn: header.
	header
		nextPutAll: 'cnonce=';
		nextPutAll: self cnonce quoted.
	
	self commaLine: header.! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/9/2004 19:51'!
commaLine: header
	header nextPut: $,; nextPutAll: String crlf.! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:02'!
domain
	^ domain! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:01'!
domain: domainCollection
	domain _ domainCollection! !

!HCDigestAuth methodsFor: 'auth accessing' stamp: 'cc 5/10/2004 00:15'!
fromPending: authToken info: infoDict
	self
		url: authToken url;
		user: authToken user;
		password: authToken password;
		realm: (infoDict at: 'realm' ifAbsent: [^nil]).

	self
		parseDomains: infoDict;
		nonce: (infoDict at: 'nonce' ifAbsent: [^nil]);
		opaque: (infoDict at: 'opaque' ifAbsent: [nil]);
		algorithm: (infoDict at: 'algorithm' ifAbsent: [nil]);
		qop: (infoDict at: 'qop' ifAbsent: [nil]).! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/11/2004 04:21'!
getA1
	| a1 |
	a1 _ self user, ':', self realm, ':', self password.
	self algorithm = 'MD5-sess'
		ifTrue: [a1 _ (self hash: a1), ':', self nonce, ':', self cnonce].

	^ a1 ! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/11/2004 04:20'!
getA2
	| a2 |
	a2 _ self method methodName, ':', self uri.
	self qop = 'auth-int' 
		ifTrue: [a2 _ a2, (self method entityBody)].
	
	^ a2! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/11/2004 04:27'!
getRequestDigest
	^ self requestDigestA1: self getA1 A2: self getA2! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/9/2004 21:00'!
hash: aString
	^ (MD5 hashMessage: aString) asString asHex asLowercase! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 20:15'!
method
	^ method! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 20:15'!
method: aMethodName
	method _ aMethodName! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:02'!
nonce
	^ nonce! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:52'!
nonceCount
	nonceCount ifNil: [nonceCount _ 0].
	^ nonceCount _ nonceCount + 1! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 21:02'!
nonceCountHeader
	^ (self nonceCount hex8 copyFrom: 4 to: 11) asLowercase! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/10/2004 00:20'!
nonceCountHeaderOn: header
	self qop ifNil: [^nil].
	self tabsOn: header.
	header 
		nextPutAll: 'nc=';
		nextPutAll: self nonceCountHeader.

	self commaLine: header.! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/10/2004 00:21'!
nonceHeaderOn: header
	self tabsOn: header.
	header
		nextPutAll: 'nonce=';
		nextPutAll: self nonce quoted.
	
	self commaLine: header.! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:02'!
nonce: aNonce
	nonce _ aNonce! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:02'!
opaque
	^ opaque! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/10/2004 00:26'!
opaqueHeaderOn: header
	self opaque ifNil: [^nil].
	self tabsOn: header.
	header
		nextPutAll: 'opaque=';
		nextPutAll: self opaque quoted.! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:03'!
opaque: anOpaqueString
	opaque _ anOpaqueString! !

!HCDigestAuth methodsFor: 'auth accessing' stamp: 'cc 5/10/2004 00:15'!
parseDomains: infoDict
	| domainString domains |
	domainString _ infoDict at: 'domain' ifAbsent: [domain _ Bag with: self url].
	domains _ domainString findTokens: ' '.
	
	self domain: (domains collect: [:dom | self url newFromRelativeText: dom withoutQuoting]).! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:06'!
qop
	^ qop! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/10/2004 00:21'!
qopHeaderOn: header
	self qop ifNil: [^nil].
	self tabsOn: header.
	header 
		nextPutAll: 'qop=';
		nextPutAll: self qop.

	self commaLine: header.! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:06'!
qop: someQoP
	| supported |
	supported _ #('auth' 'auth-int').

	(supported includes: someQoP) 
		ifTrue: [qop _ someQoP].! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/10/2004 00:21'!
realmHeaderOn: header
	self tabsOn: header.
	header
		nextPutAll: 'realm=';
		nextPutAll: self realm quoted.
	
	self commaLine: header.! !

!HCDigestAuth methodsFor: 'auth accessing' stamp: 'cc 5/10/2004 00:23'!
renderHeaderOn: header forMethod: aMethod
	self uri: aMethod url pathstring.
	self method: aMethod.

	header nextPutAll: self class type, ' '.
	self userHeaderOn: header.
	
	self 
		realmHeaderOn: header;
		nonceHeaderOn: header;
		uriHeaderOn: header;
		qopHeaderOn: header;
		cnonceHeaderOn: header;
		nonceCountHeaderOn: header;
		responseHeaderOn: header;
		opaqueHeaderOn: header.! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/11/2004 04:26'!
requestDigestA1: a1 A2: a2
	| toHash ha1 ha2 |
	ha1 _ self hash: a1.
	ha2 _ self hash: a2.
	self qop 
		ifNil: [ toHash _ ha1, ':', (self nonce, ':', ha2) ]
		ifNotNil: [toHash _ ha1, ':', (self nonce, ':', self nonceCountHeader, ':', self cnonce, ':', self qop, ':', ha2) ].
	
	^ self hash: toHash.! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/11/2004 04:27'!
responseHeaderOn: header
	self tabsOn: header.
	header
		nextPutAll: 'response=';
		nextPutAll: self getRequestDigest quoted.
	self opaque ifNotNil: [ self commaLine: header ].! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/10/2004 00:20'!
tabsOn: aWriteStream
	aWriteStream nextPutAll: '        '.! !

!HCDigestAuth methodsFor: 'authentication info' stamp: 'cc 5/11/2004 04:25'!
updateAuthInfo: infoDict
	"Follows RFC 2617 3.2.3 closely."
	| rspqop rspauth rspcnonce rspnc |
	rspqop _ infoDict at: 'qop' ifAbsent: [nil].
	rspqop 
		ifNil: [ 
			rspauth _ infoDict at: 'rspauth' ifAbsent: [nil].
			rspauth ifNotNil: [ (self authorizeRspauth: rspauth qop: nil) ifFalse: [^nil] ].
			self nonce: (infoDict at: 'nextnonce' ifAbsent: [^nil]) ]
		ifNotNil: [
			rspqop = self qop ifFalse: [^nil].
			rspauth _ infoDict at: 'rspauth' ifAbsent: [^nil].
			rspcnonce _ infoDict at: 'cnonce' ifAbsent: [^nil].
			rspnc _ infoDict at: 'nc' ifAbsent: [^nil].

			rspcnonce = self cnonce ifFalse: [^nil].
			rspnc = self nonceCountHeader ifFalse: [^nil].
			
			(self authorizeRspauth: rspauth)	 ifTrue: [
				self nonce: (infoDict at: 'nextnonce' ifAbsent: [^nil])].
			]! !

!HCDigestAuth methodsFor: 'auth accessing' stamp: 'cc 5/11/2004 04:39'!
updateFromInfo: infoDict
	self
		parseDomains: infoDict;
		nonce: (infoDict at: 'nonce' ifAbsent: [^nil]);
		opaque: (infoDict at: 'opaque' ifAbsent: [nil]);
		algorithm: (infoDict at: 'algorithm' ifAbsent: [nil]);
		qop: (infoDict at: 'qop' ifAbsent: [nil]).! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:20'!
uri
	^ uri! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/10/2004 00:30'!
uriHeaderOn: header
	self tabsOn: header.
	header
		nextPutAll: 'uri=';
		nextPutAll: self uri quoted.

	self commaLine: header.! !

!HCDigestAuth methodsFor: 'accessing' stamp: 'cc 5/9/2004 19:20'!
uri: digestUri
	uri _ digestUri! !

!HCDigestAuth methodsFor: 'header' stamp: 'cc 5/10/2004 00:17'!
userHeaderOn: header
	header
		nextPutAll: 'username=';
		nextPutAll: self user quoted.

	self commaLine: header.! !

Object subclass: #HCAuthManager
	instanceVariableNames: 'tokens pending'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Session'!

!HCAuthManager class methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 23:49'!
new
	^ super new initialize! !

!HCAuthManager methodsFor: 'updating' stamp: 'cc 5/15/2004 09:27'!
addPendingUrl: aUrl user: user password: password
	self pending add: (HCAuth new 
			user: user; 
			password: password;
			url: aUrl;
			yourself)! !

!HCAuthManager methodsFor: 'updating' stamp: 'cc 5/7/2004 19:50'!
addToken: anAuthToken
	self tokens add: anAuthToken! !

!HCAuthManager methodsFor: 'header' stamp: 'cc 5/15/2004 10:39'!
authHeaderForMethod: aMethod on: aStream
	| authtoken |
	authtoken _ self selectForMethod: aMethod ifEmpty: [^nil].
	
	aStream nextPutAll: 'Authorization: '.
	authtoken renderHeaderOn: aStream forMethod: aMethod.
	aStream nextPutAll: String crlf.! !

!HCAuthManager methodsFor: 'updating' stamp: 'cc 5/11/2004 03:39'!
distributeDigestInfo: infoDict from: aResponse
	(self tokensForUrl: aResponse url) do: [:token | token updateAuthInfo: infoDict].! !

!HCAuthManager methodsFor: 'updating' stamp: 'cc 5/11/2004 04:34'!
distributeInfo: infoDict from: response type: authType
	"If there is a pending authentication token, make it happen. Otherwise,
	it might be data for a current token."
	(HCAuth isValidType: authType) ifFalse: [^nil].

	self updatePendingFor: response url type: authType info: infoDict.
	self updateCurrentFor: response url info: infoDict.
	! !

!HCAuthManager methodsFor: 'updating' stamp: 'cc 5/15/2004 09:27'!
getInfoDictionary: rawHeader
	| infodict |
	infodict _ Dictionary new.

	(rawHeader findTokens: $,) do: [:token |
		(token findTokens: $=) pairsDo: [:name :value |
			infodict 
				at: name withBlanksTrimmed 
				put: value withBlanksTrimmed withoutQuoting
		]
	].

	^ infodict! !

!HCAuthManager methodsFor: 'updating' stamp: 'cc 5/15/2004 09:28'!
getPendingAuthsFor: aUrl
	^ self pending select: [:ptoken | ptoken url = aUrl]! !

!HCAuthManager methodsFor: 'header' stamp: 'cc 5/15/2004 09:39'!
infoFrom: aHeader
	^ (self typeInfoFrom: aHeader) last! !

!HCAuthManager methodsFor: 'header' stamp: 'cc 5/15/2004 09:26'!
initialize
	tokens _ Bag new.
	pending _ Bag new.! !

!HCAuthManager methodsFor: 'header' stamp: 'cc 5/15/2004 09:41'!
parseInfoFromResponse: aResponse
	| authtype info |
	aResponse withHeader: 'www-authenticate' do: [:authheader |
		authtype _ self typeFrom: authheader.
		info _ self getInfoDictionary: (self infoFrom: authheader).

		self distributeInfo: info from: aResponse type: authtype.
	].

	aResponse withHeader: 'authentication-info' do: [:authheader |
		info _ self getInfoDictionary: authheader.
		self distributeDigestInfo: info from: aResponse.
	].! !

!HCAuthManager methodsFor: 'header' stamp: 'cc 5/7/2004 22:02'!
pending
	^ pending! !

!HCAuthManager methodsFor: 'header' stamp: 'cc 5/15/2004 11:04'!
selectForMethod: aMethod ifEmpty: emptyBlock
	"Answers the most specific authentication token for the given method and user."
	^ self 
		selectForUrl: aMethod url 
		user: aMethod user 
		ifEmpty: emptyBlock.! !

!HCAuthManager methodsFor: 'header' stamp: 'cc 5/15/2004 11:04'!
selectForUrl: aUrl user: user ifEmpty: emptyBlock
	"Answers the most specific authentication token for the given url."
	| accepted |
	accepted _ (self tokensForUrl: aUrl) 
		select: [:auth | auth user = user].
	accepted isEmpty ifTrue: [emptyBlock value].
	
	^ accepted mostSpecific.! !

!HCAuthManager methodsFor: 'header' stamp: 'cc 5/7/2004 19:49'!
tokens
	^ tokens! !

!HCAuthManager methodsFor: 'header' stamp: 'cc 5/10/2004 23:15'!
tokensForUrl: aUrl
	^ self tokens select: [:auth | auth acceptUrl: aUrl].! !

!HCAuthManager methodsFor: 'header' stamp: 'cc 5/15/2004 09:39'!
typeFrom: aHeader
	^ (self typeInfoFrom: aHeader) first! !

!HCAuthManager methodsFor: 'header' stamp: 'cc 5/15/2004 09:38'!
typeInfoFrom: aHeader
	^ aHeader findFirstToken: ' '! !

!HCAuthManager methodsFor: 'updating' stamp: 'cc 5/10/2004 23:17'!
updateCurrentFor: aUrl info: infoDict
	(self tokensForUrl: aUrl) do: [:token |
		token updateFromInfo: infoDict.
	].! !

!HCAuthManager methodsFor: 'updating' stamp: 'cc 5/10/2004 23:10'!
updatePendingFor: aUrl type: authType info: infoDict
	(self getPendingAuthsFor: aUrl) do: [:token || newauth |
		newauth _ HCAuth fromType: authType.
		newauth fromPending: token info: infoDict.
		self addToken: newauth.
	].! !

Object subclass: #HCChunkedStream
	instanceVariableNames: 'socketstream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Network'!

!HCChunkedStream class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 15:25'!
on: anSS
	^ self new socketstream: anSS; yourself.! !

!HCChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 15:26'!
contents
	^ self upToEnd! !

!HCChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 1/20/2005 00:55'!
contents: aLength
	"HTTP/1.1 spec says we must ignore this"
	^ self upToEnd! !

!HCChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 1/20/2005 01:14'!
doesNotUnderstand: message
	^self socketstream 
		perform: message selector 
		withArguments: message arguments! !

!HCChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 1/19/2005 23:28'!
getChunkSize
	^ Number readFrom: (self socketstream upToAll: String crlf) asUppercase base: 16! !

!HCChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 1/19/2005 22:08'!
ifDisconnected: aBlock
	self socketstream isConnected ifFalse: aBlock! !

!HCChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 1/19/2005 23:33'!
skip: num
	self socketstream next: num! !

!HCChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 14:49'!
socketstream
	^socketstream! !

!HCChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 14:49'!
socketstream: anSS
	socketstream _ anSS! !

!HCChunkedStream methodsFor: 'as yet unclassified' stamp: 'cc 1/19/2005 23:29'!
upToEnd
	"Goes to the very end."
	| body chunksize |
	body _ WriteStream on: String new.
	
	[(chunksize _ self getChunkSize) = 0] whileFalse: [
		self ifDisconnected: [^nil].
		body nextPutAll: (self socketstream next: chunksize).
		self skip: 2.
	].

	^body contents! !

Object subclass: #HCClient
	instanceVariableNames: 'connection sessionManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Core'!

HCClient class
	instanceVariableNames: 'debug'!

!HCClient commentStamp: 'cc 5/11/2004 07:46' prior: 0!
For examples of my usage, see the HCFacade class comment.

Cookies and authentication will be kept around as long as you have
a reference to me. For multiple requests within the same 'session',
it is then useful to use the same client instance to execute your methods.

Most of my work is done in #executeMethod:.

Any method that is a subclass of HCMethod can be instantiated by
sending the #fromUrl: or #fromUrl:user:password: messages to their
class. A few methods have special instantiation messages in their class,
I suggest you check them out in the "Methods" category.

Authentication
-----------------
I support both Basic and Digest authentication. Depending on how you 
need authentication -

#authenticateAndExecute: is the most useful. This calls #authenticate:
and then executes the method twice. See the message comments. You only
have to send this message once for each 'protection space'.

#authenticatePreempt: is used when you know that the server is expecting
Basic authentication, and you have a large document to send. The protocol
for using it is:
client authenticatePreempt: (some method)
client executeMethod: (some method).

Cookies
--------
Cookies are handled automatically by the HCSessionManager and the HCCookieJar.!

!HCClient class methodsFor: 'as yet unclassified' stamp: 'cc 1/20/2005 00:23'!
currentVersion
	^ 'v.03'! !

!HCClient class methodsFor: 'as yet unclassified' stamp: 'cc 1/20/2005 00:20'!
debug
	^ debug ifNil: [debug _ false]! !

!HCClient class methodsFor: 'as yet unclassified' stamp: 'cc 1/16/2005 19:46'!
ifDebug: aBlock
	self debug ifTrue: aBlock! !

!HCClient class methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 21:46'!
new
	^ super new initialize! !

!HCClient class methodsFor: 'as yet unclassified' stamp: 'cc 1/20/2005 00:20'!
silent
	debug _ false! !

!HCClient class methodsFor: 'as yet unclassified' stamp: 'cc 1/20/2005 00:20'!
verbose
	debug _ true! !

!HCClient methodsFor: 'responses' stamp: 'cc 5/11/2004 06:48'!
authenticateAndExecuteMethod: aMethod
	"Authenticate this method, then execute using two passes. One to get the
	additional information required for an authentication token, the second
	to get the result."
	| auth |
	self authenticate: aMethod.
	auth _ self executeMethod: aMethod.
	"A further error occured..."
	auth response status = 401 ifFalse: [ ^auth ].
	
	^ self executeMethod: aMethod.! !

!HCClient methodsFor: 'responses' stamp: 'cc 5/11/2004 06:48'!
authenticatePreempt: aMethod
	self sessionManager authenticatePreempt: aMethod! !

!HCClient methodsFor: 'responses' stamp: 'cc 5/11/2004 06:48'!
authenticate: aMethod
	self sessionManager authenticate: aMethod! !

!HCClient methodsFor: 'as yet unclassified' stamp: 'cc 5/4/2004 05:15'!
connection
	^ connection ifNotNil: [ connection isFresh ifTrue: [ ^ connection ] ].! !

!HCClient methodsFor: 'as yet unclassified' stamp: 'cc 1/16/2005 19:49'!
connectionTo: host port: portNumber
	connection ifNil: [^self connectTo: host port: portNumber].
	connection ifStale: [^self connectTo: host port: portNumber].
	^connection! !

!HCClient methodsFor: 'as yet unclassified' stamp: 'cc 5/4/2004 23:54'!
connectTo: host port: portNumber 
	^connection := HCConnection connectToHost: host port: portNumber! !

!HCClient methodsFor: 'responses' stamp: 'cc 5/11/2004 05:07'!
errorResponse: aResponse from: aMethod stream: stream
	^ aMethod
		response: aResponse;
		instream: stream;
		yourself.! !

!HCClient methodsFor: 'testing' stamp: 'cc 1/20/2005 00:17'!
executeMethodHeadersOnly: aMethod
	"Execute the method with only headers, don't expect a response,
	and don't close the connection."
	| httpconn command |
	httpconn _ self connectTo: aMethod serverName port: aMethod serverPort.
	command _
		aMethod generateHeaders,
		self sessionHeadersFor: aMethod,
		aMethod specialHeaders.
	aMethod noContentLength: [command _ command, String crlf].
	
	httpconn sendCommand: command.
	
	HTTPClient ifDebug: [Transcript show: command; cr; cr].
! !

!HCClient methodsFor: 'responses' stamp: 'cc 1/20/2005 00:16'!
executeMethod: aMethod
	"Execute a method, send the outstream of the method, then send response for processing.
	Todo: https, (delete options trace), connect (https/http), socks, streamlining"
	| httpconn response command |
	httpconn _ self connectionTo: aMethod serverName port: aMethod serverPort.
	command _ WriteStream on: String new.
	
	"Generate headers"
	aMethod generateHeadersOn: command.
	self sessionHeadersFor: aMethod on: command.
	aMethod specialHeadersOn: command.
	
	"If method has no content length, we need a crlf"
	aMethod noContentLength: [command nextPutAll: String crlf].
	
	httpconn sendCommand: command contents.
	HCClient ifDebug: [Transcript show: command contents; cr; cr].

	self sendStreamFrom: aMethod to: httpconn.

	response _ self getResponseFrom: httpconn method: aMethod.
	self getSessionInfoFrom: response.
	^ self 
		parseResponse: response 
		from: aMethod 
		with: httpconn
! !

!HCClient methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 07:55'!
getContentsFrom: aConnection
	^ aConnection upToEnd! !

!HCClient methodsFor: 'responses' stamp: 'cc 1/10/2005 21:33'!
getResponseFrom: aConnection method: aMethod
	| header response |
	header _ aConnection upToAll: String crlfcrlf.
	response _ HCResponse fromHeader: header url: aMethod url.
	
	"100 Continue - ignore, get the next response"
	response ifStatus: 100 
		do: [^self getResponseFrom: aConnection method: aMethod].
		
	^response! !

!HCClient methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 06:12'!
getSessionInfoFrom: aResponse
	self sessionManager addSessionInfoFrom: aResponse! !

!HCClient methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 15:14'!
getStreamFrom: aConnection
	^ aConnection upToEnd! !

!HCClient methodsFor: 'responses' stamp: 'cc 1/10/2005 15:33'!
ifChunked: aResponse do: aBlock
	(aResponse header: 'transfer-encoding' equals: 'chunked') ifTrue: aBlock! !

!HCClient methodsFor: 'as yet unclassified' stamp: 'cc 1/10/2005 22:15'!
ifRedirect: aResponse do: aBlock
	^ aResponse ifStatusClass: 3 do: aBlock! !

!HCClient methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 21:45'!
initialize
	sessionManager _ HCSessionManager new.! !

!HCClient methodsFor: 'as yet unclassified' stamp: 'cc 1/10/2005 22:16'!
okToProceedWith: aResponse then: aBlock
	aResponse ifStatusClass: 2 do: aBlock! !

!HCClient methodsFor: 'responses' stamp: 'cc 1/10/2005 22:17'!
parseResponse: response from: aMethod with: aSocketStream
	| delegate stream |
	"Get the 'true' socket stream, if you will (bug found by Brian Murphy-Dye)"
	stream _ aSocketStream socketstream.
	self ifChunked: response do: [ stream _ HCChunkedStream on: stream ].

	self okToProceedWith: response then: [
		^ aMethod 
			response: response;
			instream: stream;
			yourself.
	].

	self ifRedirect: response do: [ 
		delegate _ self redirect: response from: aMethod.
		^ self 
			parseResponse: delegate response
			from: aMethod
			with: delegate instream
	].

	^ self errorResponse: response from: aMethod stream: stream

	! !

!HCClient methodsFor: 'testing' stamp: 'cc 5/5/2004 01:27'!
rawConnection
	^ connection! !

!HCClient methodsFor: 'as yet unclassified' stamp: 'cc 1/19/2005 23:49'!
redirect: response from: aMethod
	^ self executeMethod: (
		GetMethod fromUrl: (
			aMethod url newFromRelativeText: (response getHeader: 'location')))! !

!HCClient methodsFor: 'as yet unclassified' stamp: 'cc 1/20/2005 00:17'!
sendStreamFrom: aMethod to: aConnection
	| chunkSize cnts |
	"Needfully borrowed from HttpResponse>>#pvtWriteContentsOn:"
	chunkSize _ 131072.
	cnts _ aMethod outstream.
	cnts ifNil: [^nil].

	HCClient ifDebug: [Transcript cr; show: cnts contents].
	
	[cnts atEnd] whileFalse: 
		[aConnection nextPutAll: (cnts next: chunkSize); flush].

	aConnection nextPutAll: String crlf; flush.! !

!HCClient methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 03:36'!
sessionHeadersFor: aMethod
	^ self sessionManager sessionHeadersFor: aMethod! !

!HCClient methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:34'!
sessionHeadersFor: aMethod on: aStream
	self sessionManager sessionHeadersFor: aMethod on: aStream! !

!HCClient methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 21:45'!
sessionManager
	^ sessionManager! !

Object subclass: #HCCookie
	instanceVariableNames: 'attributes url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Session'!

!HCCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 06:06'!
addAttribute: name value: value
	| realname |
	realname _ name.
	name = 'expires' ifTrue: [realname _ 'max-age'].
	self attributes at: realname put: value.! !

!HCCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:28'!
attributes
	^ attributes ifNil: [ attributes _ Dictionary new ]! !

!HCCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 06:19'!
domain
	^ self attributes at: 'domain' ifAbsent: [self url authority]! !

!HCCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 06:06'!
expires
	"Backwards-compatibility"
	^ self maxage! !

!HCCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:29'!
maxage
	^ self attributes at: 'max-age' ifAbsent: [nil]! !

!HCCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:29'!
maxageTimeStamp
	self expires ifNil: [^ TimeStamp current plusDays: 1].
	^ TimeStamp readFrom: (self expires fromFirstDigit) readStream! !

!HCCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:29'!
name
	^ self attributes at: 'name'! !

!HCCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:30'!
path
	^ self attributes at: 'path' ifAbsent: [ '\' ].! !

!HCCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:30'!
secure
	^ attributes at: 'secure' ifAbsent: [nil]! !

!HCCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 06:04'!
url
	^url! !

!HCCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 06:04'!
url: aUrl
	url _ aUrl! !

!HCCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:30'!
value
	^ self attributes at: 'value'! !

!HCCookie methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:32'!
version
	^ self attributes at: 'version' ifAbsent: [^nil]! !

Object subclass: #HCCookieJar
	instanceVariableNames: 'cookies tempurl'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Session'!

!HCCookieJar methodsFor: 'parsing' stamp: 'ctc 11/18/2005 02:31'!
addAttributesFrom: parameters to: aCookie
	| namedparams tokens |
	namedparams := #('max-age' 'expires' 'domain' 'path' 'secure' 'comment').
	tokens := self getTokensFrom: parameters.

	(namedparams includes: tokens first asLowercase) 
	ifTrue: [
		aCookie
			addAttribute: tokens first asLowercase
			value: tokens last] 
	ifFalse: [ 
		aCookie attributes 
			at: 'name' put: tokens first;
			at: 'value' put: tokens last.].! !

!HCCookieJar methodsFor: 'adding' stamp: 'cc 5/5/2004 06:33'!
addCookie: aCookie
	"Check for expiration date"
	(self checkExpirationForAndRemove: aCookie) ifTrue: [^nil].
	"If we wish to delete a cookie..."
	(self checkForDeletion: aCookie) ifTrue: [^nil].
	"Overwriting cookies"
	self checkForOverride: aCookie.
	self cookies add: aCookie! !

!HCCookieJar methodsFor: 'gathering' stamp: 'cc 5/5/2004 06:33'!
checkExpirationForAndRemove: aCookie
	(self checkForExpiration: aCookie) ifTrue: [ self removeCookie: aCookie. ^true ].
	^false! !

!HCCookieJar methodsFor: 'gathering' stamp: 'cc 5/5/2004 06:30'!
checkExpirations
	self cookies do: [:cookie | 
		(self checkForExpiration: cookie) ifTrue: [ self removeCookie: cookie ]
	].! !

!HCCookieJar methodsFor: 'adding' stamp: 'cc 1/16/2005 20:07'!
checkForDeletion: aCookie
	(self checkForExpiration: aCookie) | (aCookie maxage = 0) 
	ifTrue: [ self cookies do: [:cookie | 
		aCookie name = cookie name 
			ifTrue: [ self cookies remove: cookie. ^true ]]].
	^ false! !

!HCCookieJar methodsFor: 'adding' stamp: 'cc 5/5/2004 04:19'!
checkForExpiration: aCookie
	^ aCookie maxageTimeStamp < HCDateAdapter GMTStamp! !

!HCCookieJar methodsFor: 'adding' stamp: 'cc 1/17/2005 01:30'!
checkForOverride: aCookie
	self cookies do: [:cookie |
		(aCookie name = cookie name) 
			& (aCookie path = cookie path) 
			& (aCookie domain = cookie domain) 
		ifTrue: [self cookies remove: cookie]
	].! !

!HCCookieJar methodsFor: 'gathering' stamp: 'cc 5/15/2004 10:01'!
cookieHeaderForUrl: aUrl on: aStream
	| validCookies |
	validCookies _ self cookiesForUrl: aUrl.
	validCookies isEmpty ifTrue: [^nil].
	aStream nextPutAll: 'Cookie: '.
	
	validCookies 
		do: [:cookie | aStream nextPutAll: cookie name, '=', cookie value]
		separatedBy: [aStream nextPutAll: '; '].

	aStream nextPutAll: String crlf.
	! !

!HCCookieJar methodsFor: 'gathering' stamp: 'cc 5/5/2004 05:11'!
cookies
	^ cookies ifNil: [cookies _ Bag new]! !

!HCCookieJar methodsFor: 'gathering' stamp: 'cc 5/5/2004 06:30'!
cookiesForUrl: aUrl
	| root path domainValid pathValid |
	root _ aUrl authority.
	path _ aUrl page.

	"Check expirations"
	self checkExpirations.

	"Domain"
	domainValid _ self cookies select: [:cookie || domain |
		domain _ (cookie domain beginsWith: '.') 
			ifTrue: [cookie domain copyFrom: 2 to: cookie domain size]
			ifFalse: [cookie domain].
 		root includesSubString: domain].

	"Path"
	pathValid _ domainValid select: [:cookie | path includesSubString: cookie path].

	^ pathValid! !

!HCCookieJar methodsFor: 'parsing' stamp: 'cc 5/5/2004 06:16'!
getCookieLines: aHeader
	| token |
	token _ 'Set-Cookie:'.
	^ aHeader 
		select: [:line | line includesSubString: token]
		thenCollect: [:line | line copyFrom: token size + 1 to: line size]! !

!HCCookieJar methodsFor: 'parsing' stamp: 'ctc 11/18/2005 02:35'!
getTokensFrom: parameters
	^ (parameters includesSubString: '=')
		ifTrue: [parameters withBlanksTrimmed findFirstToken: #=]
		ifFalse: [
			(OrderedCollection new)
				add: parameters;
				add: '';
				yourself]! !

!HCCookieJar methodsFor: 'parsing' stamp: 'cc 5/5/2004 06:02'!
newCookieFromLine: aLine
	| params newcookie |
	newcookie _ HCCookie new.
	params _ aLine findTokens: #;.
	
	params do: [:p | self addAttributesFrom: p to: newcookie].
	newcookie url: self tempurl.
	^ newcookie! !

!HCCookieJar methodsFor: 'parsing' stamp: 'cc 5/13/2004 02:59'!
parseCookie: aCookieLine
	self addCookie: (self newCookieFromLine: aCookieLine).! !

!HCCookieJar methodsFor: 'parsing' stamp: 'cc 5/13/2004 02:59'!
parseInfoFromResponse: aResponse
	| lines |
	self tempurl: aResponse url.
	lines _ self getCookieLines: aResponse rawHeaders.
	lines do: [:line | self parseCookie: line].
	self tempurl: nil.! !

!HCCookieJar methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 04:22'!
removeCookie: aCookie
	self cookies remove: aCookie! !

!HCCookieJar methodsFor: 'parsing' stamp: 'cc 5/5/2004 06:01'!
tempurl
	^tempurl! !

!HCCookieJar methodsFor: 'parsing' stamp: 'cc 5/5/2004 06:01'!
tempurl: aUrl
	tempurl _ aUrl! !

Object subclass: #HCDateAdapter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Core'!

!HCDateAdapter class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 04:17'!
fromGMT
	"Author is in the Pacific time zone, so that's -8 hours from GMT.
	Please set this accordingly."
	^ (-8)! !

!HCDateAdapter class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 04:19'!
GMTStamp
	^ self localTimeStampToGMT! !

!HCDateAdapter class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 04:16'!
localTimeStampToGMT
	"Everything in HTTPClient is stored in GMT, but sometimes you have to compare
	against current system time.
	See #fromGMT and #toGMT"
	^ TimeStamp current plusSeconds: (self toGMT * 3600)! !

!HCDateAdapter class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 04:15'!
toGMT
	^ self fromGMT negated! !

Object subclass: #HCFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Core'!

!HCFacade commentStamp: 'cc 5/11/2004 07:34' prior: 0!
My class methods are similar to those in HTTPSocket, to show examples of
how to do things with HTTPClient. 

Of particular interest are the similarities between httpGet, httpPost, httpPut, 
and httpPostMultipart. HTTPClient allows these simple methods to be up to an 
order of magnitude (in the case of multipart) shorter in code length.!

!HCFacade class methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 07:18'!
httpGetDocument: url
	^ self httpGet: url accept: 'application/octet-stream'.! !

!HCFacade class methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 06:01'!
httpGet: urlString
	^ self httpGet: urlString accept: '*/*'! !

!HCFacade class methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 06:02'!
httpGet: url accept: mimeType
	| client get |
	client _ HCClient new.
	get _ GetMethod fromUrl: url asUrl.
	get addMimetype: mimeType.
	client executeMethod: get.

	^ get MIMEDocument! !

!HCFacade class methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 07:16'!
httpGet: url user: user password: password
	| client get |
	client _ HCClient new.
	get _ GetMethod fromUrl: url asUrl user: user password: password.
	get addMimetype: '*/*'.
	client authenticateAndExecuteMethod: get.

	^ get MIMEDocument! !

!HCFacade class methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 07:24'!
httpGif: url
	| doc ggg stream |
	doc _ self httpGet: url asUrl accept: 'image/gif'.
	doc contentType = 'image/gif' ifFalse: [
		self inform: 'An error occured.'.
		^ ColorForm extent: 20@20 depth: 8].
	stream _ RWBinaryOrTextStream with: doc contents.
	stream binary; reset.
	(ggg _ GIFReadWriter new) setStream: stream.
	^ ggg nextImage.! !

!HCFacade class methodsFor: 'as yet unclassified' stamp: 'cc 5/17/2004 02:12'!
httpJpeg: url
	| doc ggg stream |
	doc _ self httpGet: url asUrl accept: 'image/jpeg'.
	doc contentType = 'image/jpeg' ifFalse: [
		self inform: 'An error occured.'.
		^ ColorForm extent: 20@20 depth: 8].
	stream _ RWBinaryOrTextStream with: doc contents.
	stream binary; reset.
	(ggg _ JPEGReadWriter new) setStream: stream.
	^ ggg nextImage.! !

!HCFacade class methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 06:02'!
httpPostDocument: url args: argsDict
	^ self httpPostDocument: url asUrl args: argsDict accept: 'application/octet-stream'! !

!HCFacade class methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 06:02'!
httpPostDocument: url args: argsDict accept: mimeType
	| client post |
	client _ HCClient new.
	post _ PostMethod fromUrl: url asUrl args: argsDict.
	post addMimetype: mimeType.
	client executeMethod: post.

	^ post MIMEDocument! !

!HCFacade class methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 06:02'!
httpPostMultipart: url args: argsDict accept: mimeType
	"argsDict should be a dictionary containing name -> mimeDocument pairs.
	See the comment in MultipartPostMethod>>#printName:document:on:"
	| client multi |
	client _ HCClient new.
	multi _ MultipartPostMethod fromUrl: url asUrl args: argsDict.
	multi addMimetype: mimeType.
	client executeMethod: multi.

	^ multi MIMEDocument! !

!HCFacade class methodsFor: 'as yet unclassified' stamp: 'cc 5/17/2004 02:11'!
httpPost: url args: argsDict accept: mimeType
	^ self httpPostDocument: url args: argsDict accept: mimeType! !

!HCFacade class methodsFor: 'as yet unclassified' stamp: 'cc 5/17/2004 02:09'!
httpPost: url args: argsDict user: user password: password
	| client post |
	client _ HCClient new.
	post _ PostMethod 
		fromUrl: url asUrl 
		user: user 
		password: password.
	post arguments: argsDict.

	client authenticateAndExecuteMethod: post.

	^ post MIMEDocument! !

!HCFacade class methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 06:02'!
httpPut: stream to: url user: user password: password
	"Upload the contents of the stream to a file on the server, we use
	authenticatePreempt to preemptively send basic authentication."
	| client put |
	client _ HCClient new.
	put _ PutMethod 
		fromUrl: url asUrl
		stream: stream
		user: user 
		password: password.

	client 
		authenticatePreempt: put;
		executeMethod: put.

	^ put MIMEDocument! !

Object subclass: #HCMethod
	instanceVariableNames: 'url acceptedMimetypes contents instream outstream response user password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Core'!

HCMethod subclass: #GetMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Methods'!

!GetMethod class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 16:48'!
retrieveContentsFrom: aUrl on: aClient
	| get |
	get _ self fromUrl: aUrl.
	aClient executeMethod: get.
	^ get MIMEDocument contents! !

!GetMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 02:21'!
methodName
	^ 'GET'! !

!HCMethod class methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 19:20'!
fromMethod: aMethod url: aUrl
	^ self fromUrl: aUrl! !

!HCMethod class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 13:54'!
fromUrl: aUrl
	^ self new url: aUrl; yourself.! !

!HCMethod class methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 23:53'!
fromUrl: aUrl user: user password: password
	^ self new 
		url: aUrl; 
		user: user; 
		password: password; 
		yourself.! !

!HCMethod class methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 02:00'!
httpVersion
	^ 'HTTP/1.1'! !

!HCMethod class methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 06:15'!
new
	^ super new initialize! !

!HCMethod class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 00:48'!
userAgentString
	^ 'User-Agent: ',
		'HTTPClient (Squeak)', '-', 
		HCClient currentVersion.! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 02:30'!
acceptedMimetypes
	^ acceptedMimetypes ifNil: [ acceptedMimetypes _ Bag with: 'text/html' ]! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/5/2004 02:18'!
addMimetype: aMimetype 
	self acceptedMimetypes add: aMimetype! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 1/20/2005 00:02'!
contents
	^contents ifNil: [
		self instream closeAfter: [
			^contents _ self response 
				headerExists: 'content-length'
				do: [:length | self instream contents: length asNumber]
				otherwise: [self instream contents]]]
	! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/9/2004 20:35'!
entityBody
	self subclassResponsibility! !

!HCMethod methodsFor: 'private' stamp: 'cc 5/15/2004 10:04'!
generateHeadersOn: aStream
	"Generate the non-session headers for this method"
	self requestHeaderOn: aStream.
	self hostHeaderOn: aStream.
	self userAgentHeaderOn: aStream.
	self mimeTypeHeaderOn: aStream.! !

!HCMethod methodsFor: 'private' stamp: 'cc 5/15/2004 06:32'!
hostHeaderOn: aStream
	aStream nextPutAll: self hostString; nextPutAll: String crlf.! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 02:48'!
hostString
	^ 'Host: ', self serverName! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/11/2004 06:15'!
initialize! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/11/2004 05:05'!
instream
	^ instream! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/11/2004 05:05'!
instream: aStream
	instream _ aStream! !

!HCMethod methodsFor: 'httpadapter' stamp: 'cc 5/15/2004 10:12'!
MIMEDocument
	| type |
	(self url path last = '') ifFalse: [
		type _ MIMEDocument guessTypeFromName: self url path last].

	"The server probably knows better than us."
	self response withHeader: 'content-type' do: [:mimetype | type _ mimetype].

	^MIMEDocument
		contentType: type
		content: self contents
		url: self url! !

!HCMethod methodsFor: 'private' stamp: 'cc 5/3/2004 02:20'!
methodName
	self subclassResponsibility! !

!HCMethod methodsFor: 'private' stamp: 'cc 5/15/2004 10:08'!
mimeTypeHeaderOn: aStream

	aStream nextPutAll: 'ACCEPT: '.
	self acceptedMimetypes 
		do: [:type | aStream nextPutAll: type]
		separatedBy: [aStream nextPutAll: ' '].

	aStream nextPutAll: String crlf.! !

!HCMethod methodsFor: 'httpadapter' stamp: 'cc 1/16/2005 19:44'!
noContentLength: aBlock
	"This method does not have a content length and must do something special,
	usually a crlf"
	aBlock value.! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/11/2004 05:07'!
outstream
	^ outstream! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/11/2004 05:07'!
outstream: aStream
	outstream _ aStream! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/7/2004 20:02'!
page
	^ self url pageAndQuery! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/7/2004 23:52'!
password
	^ password! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/7/2004 23:52'!
password: aPassword
	password _ aPassword! !

!HCMethod methodsFor: 'private' stamp: 'cc 5/15/2004 07:05'!
requestHeaderOn: aStream
	aStream
		nextPutAll: self methodName;
		nextPutAll: ' ';
		nextPutAll: self page;
		nextPutAll: ' ';
		nextPutAll: self class httpVersion;
		nextPutAll: String crlf.! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 19:31'!
response
	^ response! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 19:31'!
response: aResponse
	response _ aResponse! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 02:14'!
serverName
	^ self url authority! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 02:16'!
serverPort
	^ self url port ifNil: [80]! !

!HCMethod methodsFor: 'private' stamp: 'cc 5/15/2004 10:34'!
specialHeadersOn: aStream
	^nil! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 02:02'!
url
	^ url! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/3/2004 02:02'!
url: aUrl
	url _ aUrl! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/7/2004 23:52'!
user
	^ user! !

!HCMethod methodsFor: 'private' stamp: 'cc 5/15/2004 06:33'!
userAgentHeaderOn: aStream
	aStream nextPutAll: self class userAgentString; nextPutAll: String crlf.! !

!HCMethod methodsFor: 'accessing' stamp: 'cc 5/7/2004 23:52'!
user: aUser
	user _ aUser! !

HCMethod subclass: #HeadMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Methods'!

!HeadMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 05:33'!
methodName
	^ 'HEAD'! !

HCMethod subclass: #MultipartPostMethod
	instanceVariableNames: 'boundary documents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Methods'!

!MultipartPostMethod class methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:54'!
fromUrl: aUrl args: argDict
	^ (self fromUrl: aUrl) documents: argDict; yourself! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 05:54'!
addDocument: aMIMEDocument named: name
	self documents at: name put: aMIMEDocument! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 06:17'!
contentLength
	^ (self documents values collect: [:doc | doc contentStream size]) sum asString.! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:13'!
contentLengthHeaderOn: header
	header
		nextPutAll: 'Content-Length: ';
		nextPutAll: self contentLength;
		nextPutAll: String crlf.! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 05:44'!
contentType
	^ 'multipart/form-data'! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 05:47'!
contentTypeHeader
	^ 'Content-Type: ', self contentType, '; ', 'boundary=', self mimeBoundary! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:15'!
contentTypeHeaderOn: header
	header
		nextPutAll: 'Content-Type: ';
		nextPutAll: self contentType;
		nextPutAll: '; boundary=';
		nextPutAll: self mimeBoundary;
		nextPutAll: String crlf.! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 05:48'!
documents
	^ documents! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:54'!
documents: aDictionary
	documents _ aDictionary! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 05:54'!
initialize
	documents _ Dictionary new.! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 05:43'!
methodName
	^ 'POST'! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 06:17'!
mimeBoundary
	^ boundary ifNil: [
		boundary _ (MD5 hashMessage: 
			TimeStamp current asSeconds asString) asString asHex asLowercase]! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 1/16/2005 19:57'!
noContentLength: aBlock
	"Do nothing - we have a content length"! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 06:34'!
outstream
	| ws startBoundary endBoundary |
	ws _ WriteStream on: String new.
	startBoundary _ '--', self mimeBoundary.
	endBoundary _ '--', self mimeBoundary, '--'.

	self documents keysAndValuesDo: [:name :doc |
		ws nextPutAll: startBoundary, String crlf.
		self printName: name document: doc on: ws.
		ws nextPutAll: String crlf.
	].
	ws nextPutAll: endBoundary.
	^ ws contents readStream! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 07:30'!
printName: name document: document on: writestream
	"Documents must be of MIMEDocument type - or a facade thereof."
	writestream 
		nextPutAll: 'Content-disposition: multipart/form-data; name=';
		nextPutAll: name.
	(document url notNil 
		and: [(document url path last = '') not])
			ifTrue: [writestream nextPutAll: ' filename=', document url path last].
	writestream nextPutAll: String crlf.

	writestream 
		nextPutAll: 'Content-Type: ';
		nextPutAll: document contentType;
		nextPutAll: String crlfcrlf.

	writestream nextPutAll: document contents.! !

!MultipartPostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:24'!
specialHeadersOn: header
	self contentTypeHeaderOn: header.
	self contentLengthHeaderOn: header.! !

HCMethod subclass: #PostMethod
	instanceVariableNames: 'arguments contentType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Methods'!

!PostMethod class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 00:05'!
fromUrl: aUrl args: argDict
	^ (self fromUrl: aUrl) arguments: argDict; yourself! !

!PostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 03:08'!
addArgument: name Value: value
	self arguments at: name put: value! !

!PostMethod methodsFor: 'as yet unclassified' stamp: 'cc 1/16/2005 19:55'!
argString
	"Return the args in a long string, as encoded in a url"
	| argsString |
	argsString _ WriteStream on: String new.
	
	self arguments associations do: [ :assoc |
			argsString nextPutAll: assoc key encodeForHTTP.
			argsString nextPut: $=.
			argsString nextPutAll: assoc value encodeForHTTP. ]
		separatedBy: [ argsString nextPut: $& ].

	^ argsString contents! !

!PostMethod methodsFor: 'as yet unclassified' stamp: 'cc 1/16/2005 19:55'!
argStringHeaderOn: header
	header nextPutAll: self argString; nextPutAll: String crlf.! !

!PostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 03:07'!
arguments
	^ arguments ifNil: [ arguments _ Dictionary new ]! !

!PostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 19:00'!
arguments: args
	arguments _ args! !

!PostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 05:45'!
contentLength
	^ self argString size printString! !

!PostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:18'!
contentLengthHeaderOn: header
	header
		nextPutAll: 'Content-Length: ';
		nextPutAll: self contentLength;
		nextPutAll: String crlf.! !

!PostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 05:44'!
contentType
	^ contentType ifNil: [ contentType _ 'application/x-www-form-urlencoded' ]! !

!PostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:19'!
contentTypeHeaderOn: header
	header
		nextPutAll: 'Content-Type: ';
		nextPutAll: self contentType;
		nextPutAll: String crlf.! !

!PostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 19:00'!
contentType: aType
	contentType _ aType! !

!PostMethod methodsFor: 'as yet unclassified' stamp: 'cc 1/16/2005 19:56'!
getArgument: name
	^self arguments at: name ifAbsent: [ nil ]! !

!PostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 03:02'!
methodName
	^ 'POST'! !

!PostMethod methodsFor: 'as yet unclassified' stamp: 'cc 1/16/2005 19:54'!
noContentLength: aBlock
	"Do nothing, we have a content length"! !

!PostMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:24'!
specialHeadersOn: header
	
	self contentTypeHeaderOn: header.
	self contentLengthHeaderOn: header.
	header nextPutAll: String crlf.

	self argStringHeaderOn: header.
! !

HCMethod subclass: #PutMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Methods'!

!PutMethod class methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 05:43'!
fromUrl: aUrl stream: aStream user: user password: password
	^ (self fromUrl: aUrl user: user password: password) outstream: aStream; yourself.! !

!PutMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 06:42'!
contentLength
	^ self outstream size asString! !

!PutMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:24'!
contentLengthHeaderOn: header
	header
		nextPutAll: 'Content-Length: ';
		nextPutAll: self contentLength;
		nextPutAll: String crlf.! !

!PutMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/11/2004 05:41'!
methodName
	^ 'PUT'! !

!PutMethod methodsFor: 'as yet unclassified' stamp: 'cc 1/16/2005 19:58'!
noContentLength: aBlock
	"Do nothing - there's a content length"! !

!PutMethod methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:21'!
specialHeadersOn: header
	self contentLengthHeaderOn: header.! !

Object subclass: #HCResponse
	instanceVariableNames: 'headers status version rawHeaders url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Core'!

!HCResponse class methodsFor: 'as yet unclassified' stamp: 'cc 1/10/2005 22:23'!
foldLines: lines
	| newLines |
	newLines _ OrderedCollection new.
	lines do: [ :line |
		line first isSeparator 
			ifTrue: [newLines at: newLines size  put: (newLines last, line) ]
			ifFalse: [ newLines add: line ] ].

	^ newLines! !

!HCResponse class methodsFor: 'as yet unclassified' stamp: 'cc 1/20/2005 00:18'!
fromHeader: aHeader url: aUrl
	| lines newResponse statusLine |
	HCClient ifDebug: [Transcript cr; show: aHeader; cr].
	
	newResponse := self fromUrl: aUrl.
	lines := aHeader findTokens: String crlf.
	statusLine := lines first.
	
	newResponse version: (statusLine findTokens: ' ') first.
	newResponse status: (statusLine findTokens: ' ') second asInteger.
	lines := lines copyFrom: 2 to: lines size.
	lines := self foldLines: lines.
	newResponse rawHeaders: lines.
	lines do: [:line || i |
		i := line indexOf: $:.
		i > 0 ifTrue: 
			[newResponse addHeader: (line copyFrom: 1 to: i - 1) asLowercase
				Value: (line copyFrom: i + 1 to: line size) withBlanksTrimmed]].
	^newResponse! !

!HCResponse class methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 05:58'!
fromUrl: aUrl
	^ self new url: aUrl; yourself.! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 07:17'!
addHeader: headerName Value: headerValue
	self headers at: headerName put: headerValue! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 07:16'!
getHeader: headerName
	^ self headers at: headerName ifAbsent: [nil]! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 1/10/2005 22:25'!
headerExists: headerName
	self headers at: headerName ifAbsent: [^false].
	^true! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 1/20/2005 00:33'!
headerExists: headerName do: aHeaderBlock otherwise: somethingElse
	^(self headerExists: headerName)
		ifTrue: [aHeaderBlock value: (self getHeader: headerName)]
		ifFalse: somethingElse! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 07:14'!
headers
	^ headers ifNil: [ headers _ Dictionary new ]! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 1/10/2005 15:26'!
header: headerName equals: aValue
	^ (self headers at: headerName ifAbsent: [^false]) = aValue! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 1/10/2005 21:31'!
ifStatusClass: aNumber do: aBlock
	(self statusClass = aNumber) ifTrue: aBlock! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 1/10/2005 21:31'!
ifStatus: aNumber do: aBlock
	(self status = aNumber) ifTrue: aBlock! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 03:29'!
rawHeaders
	^ rawHeaders! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 03:28'!
rawHeaders: aCollection
	rawHeaders _ aCollection! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 07:14'!
status
	^ status! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 07:15'!
statusClass
	^ self status // 100! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 07:14'!
status: aStatus
	status _ aStatus! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 05:58'!
url
	^url! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 05:57'!
url: aUrl
	url _ aUrl! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 07:18'!
version
	^ version! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 5/3/2004 07:18'!
version: anHttpVersion
	version _ anHttpVersion! !

!HCResponse methodsFor: 'as yet unclassified' stamp: 'cc 1/19/2005 23:51'!
withHeader: headerName do: aBlock
	^ self headerExists: headerName do: aBlock otherwise: [nil]! !

Object subclass: #HCSessionManager
	instanceVariableNames: 'cookiejar authmanager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Session'!

!HCSessionManager class methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 21:44'!
new
	^ super new initialize! !

!HCSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 05:14'!
addLine: aHeader to: aWriteStream
	aHeader = '' ifTrue: [^nil].
	aWriteStream nextPutAll: aHeader; nextPutAll: String crlf.! !

!HCSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 21:40'!
addSessionInfoFrom: aResponse
	self cookiejar 
		parseInfoFromResponse: aResponse.
	self authmanager
		parseInfoFromResponse: aResponse.! !

!HCSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 05:48'!
authenticatePreempt: aMethod
	| newauth |
	newauth _ HCBasicAuth new 
		user: aMethod user; 
		password: aMethod password;
		url: aMethod url;
		yourself.
	self authmanager addToken: newauth.! !

!HCSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/8/2004 00:00'!
authenticate: aMethod
	self authmanager 
		addPendingUrl: aMethod url 
		user: aMethod user 
		password: aMethod password.! !

!HCSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 21:37'!
authmanager
	^ authmanager! !

!HCSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 21:37'!
cookiejar
	^ cookiejar! !

!HCSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/7/2004 21:37'!
initialize
	cookiejar _ HCCookieJar new.
	authmanager _ HCAuthManager new.! !

!HCSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:01'!
sessionHeadersFor: aMethod
	| header |
	header _ WriteStream on: String new.
	
	self cookiejar cookieHeaderForUrl: aMethod url on: header.
	self authmanager authHeaderForMethod: aMethod on: header.

	^ header contents
! !

!HCSessionManager methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:03'!
sessionHeadersFor: aMethod on: header
	self cookiejar cookieHeaderForUrl: aMethod url on: header.
	self authmanager authHeaderForMethod: aMethod on: header.! !

!String methodsFor: '*HTTPClient' stamp: 'cc 5/5/2004 12:08'!
findFirstToken: aToken
	"Finds the first occurance of aToken, and answeres a collection of everything before and after."
	| index tokens |
	(self includesSubString: aToken) ifFalse: [^ self].
	index _ self findString: aToken.
	tokens _ OrderedCollection new.

	^ tokens
		add: (self copyFrom: 1 to: index - 1);
		add: (self copyFrom: index + aToken size to: self size);
		yourself.! !

!String methodsFor: '*HTTPClient' stamp: 'cc 4/27/2004 07:09'!
fromFirstDigit
	| index |
	index _ 1.
	[ (self at: index) isDigit ] whileFalse: [ index _ index + 1 ].
	
	^ self copyFrom: index to: self size! !

!String methodsFor: '*HTTPClient' stamp: 'cc 5/9/2004 19:28'!
quoted
	"Answers the receiver with a copy of self surrounded by double quotes."
	^ $" asString, self, $" asString! !

!Number class methodsFor: '*HTTPClient' stamp: 'cc 5/5/2004 07:51'!
toDecimal: aString fromBase: aNumber
	| value numberline |
	numberline _ '0123456789abcdefghijklmnopqrstuvwxyz' truncateTo: aNumber.
	! !

!Collection methodsFor: '*HTTPClient' stamp: 'cc 5/15/2004 11:04'!
mostSpecific
	"Forwarded call for clarity. Implemented #< and #max: in HCAuth."
	^ self max! !

TestCase subclass: #HTTPMethodTests
	instanceVariableNames: 'listener client serverurl get head post'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HTTPClient-Tests'!

!HTTPMethodTests methodsFor: 'as yet unclassified' stamp: 'cc 5/15/2004 10:42'!
donttestGetMethod
	| headers |
	get _ GetMethod fromUrl: serverurl.
	client executeMethodHeadersOnly: get.
	headers _ listener receiveData.

	self assert: (headers includesSubString: 'GET / HTTP/1.1').
	self assert: (headers includesSubString: 'Host: 127.0.0.1').! !

!HTTPMethodTests methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:08'!
setUp
	client _ HCClient new.
	serverurl _ 'http://127.0.0.1:7357' asUrl.
	listener _ Socket newTCP listenOn: 7357.! !

!HTTPMethodTests methodsFor: 'as yet unclassified' stamp: 'cc 5/5/2004 02:12'!
tearDown
	listener closeAndDestroy.
	client rawConnection close.! !
